;
; TransMSX v1.2
;-=============-
;
; This is link-software to send and recieve files from an MSX to a TI-83/82
; calculator (Texas Instruments) and vice versa.
; You'll have to use the cable described in the textfile connected to
; joystickport 1.
;
; Code written and assembled in CTNG's Compass #1.2.09
;
;
;(c) 1999  Grauw / Datax
;
;
;ToDo:
;- Add handling of checksum errors...
;- Add joystickport 2-support... (on request?)
;- Make a nice GUI.
;
;
	org	#100
;
Bdos:	equ	5
FCB1:	equ	#5C
COMMANDLINE:	equ	#80
Buffer_Start:	equ	#4000	;Keep in mind Buf_St-2 is also used...
CMD_VAR:	equ	#06	;Variable header
CMD_WAITING:	equ	#09	;Ready to recieve data sign
CMD_DATA:	equ	#15	;Data packet
CMD_REFUSE:	equ	#36	;Refused
CMD_ACK:	equ	#56	;Ack
CMD_CHKERR:	equ	#5A	;Checksum error, resend
CMD_EOT:	equ	#92	;End Of Transmission
CMD_SENDVAR:	equ	#C9	;Send( variable header
s00:	equ	%00001100
s01:	equ	%00001101	;Send values
s10:	equ	%00001110
s11:	equ	%00001111
r00:	equ	%00000000	;Recieve comparisation values
r01:	equ	%00010000	;(after AND r11)
r10:	equ	%00100000
r11:	equ	%00110000
;
Send_TI:	MACRO	@value
	ld	a,15
	out	(#A0),a
	ld	a,@value
	out	(#A1),a
	ENDM
Recv_TIa:	MACRO
	ld	a,14
	out	(#A0),a
	ENDM
Recv_TIb:	MACRO
	in	a,(#A2)
	ENDM

SetTestESC:	MACRO
	in	a,(#aa)
	and	#F0
	or	7
	out	(#aa),a
	ENDM
DoTestESC:	MACRO
	in	a,(#a9)
	bit	2,a
	ENDM

TestFileType:	MACRO	@char
	ld	a,(hl)
	inc	hl
	cp	@char
	jp	nz,Xmit_NoTI83
	ENDM

;
;******************************************************************************
;*** Start of TransMSX!!! *****************************************************
;******************************************************************************
;
;
Start:	ld	(save_SP),sp	;Save StackPointer
	ld	hl,Buffer_Start
	ld	(RECVBUFFER),hl	;Reset recievebuffer startadress
	ld	de,INTROTEXT
	ld	c,9
	call	Bdos
;
	ld	a,(COMMANDLINE)	;Check commandlinelength...
	cp	0	;If 0, then RECIEVE.
	jp	z,Recieve_File
	jp	Send_File



;
;==============================================================================
;= Send files... ==============================================================
;==============================================================================
;
;  START SENDING VAR-HEADER
;  ========================
Send_File:	xor	a
	ld	(NR_OF_VARS),a	;Reset nr. of vars-counter
	call	Load_VarFile
	ld	de,(DATALENGTH)	;BC = length of entire datapart
	ld	hl,Buffer_Start
	add	hl,de
	ld	(DATAEND),hl
SendLoop:	ld	hl,(RECVBUFFER)	;Check if end of vars...
	ld	de,(DATAEND)
	and	a
	sbc	hl,de
	jp	nc,Send_End
;
	ld	hl,(RECVBUFFER)
	ld	bc,4
	add	hl,bc
	ld	a,(hl)
	ld	(CUR_VARTYPE),a
	call	SPrt_83Name	;Print VARname
;
	ld	a,CMD_VAR	;Send VAR-header
	call	Send_Packet

	call	Recv_Header	;Wait for ACKnowledgement...
	cp	CMD_CHKERR
	jp	z,Chksum_Error
	cp	CMD_ACK
	jp	nz,Xmit_Error
	call	Recv_Header
	cp	CMD_REFUSE
	jp	z,S_Refused
	cp	CMD_WAITING
	jp	nz,Xmit_Error


;             START SENDING VAR-DATA
;             ======================
	ld	a,CMD_ACK
	ld	de,0
	call	Send_Header
	ld	a,CMD_DATA
	call	Send_Packet

	call	Recv_Header	;Wait for ACKnowledgement...
	cp	CMD_CHKERR
	jp	z,Chksum_Error
	cp	CMD_ACK
	jp	nz,Xmit_Error
	ld	a,(CUR_VARTYPE)
	cp	#13	;Check if the VAR was a backup...
	jp	nz,SendLoop	;If not, repeat the mainloop.

;             IF BACKUP, SEND ANOTHER BLOCK
;             =============================
	ld	a,CMD_DATA	;Send another datablock
	call	Send_Packet
	call	Recv_Header	;Wait for ACKnowledgement...
	cp	CMD_CHKERR
	jp	z,Chksum_Error
	cp	CMD_ACK
	jp	nz,Xmit_Error

	ld	a,CMD_DATA	;Send another datablock
	call	Send_Packet
;
	ld	de,BACKUPSENT
	ld	c,9	;Display 'Backup sent' message...
	call	Bdos
	jp	Send_E_NoEOT


;
;Jump here in case a REFUSE-header joined the party...
;
S_Refused:	call	RecvByte	;Recieve the "Refused"-code
	push	af	;
	call	RecvByte	;(skip checksum)
	call	RecvByte
	pop	af
	cp	1	;1=Quit
	jp	z,S_Quit
	cp	2	;2=File skipped
	jp	z,S_Skipped
	jp	Xmit_Error


;
;TI-83 told to skip file...
;
S_Skipped:	ld	hl,(RECVBUFFER)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	add	hl,de
	ld	(RECVBUFFER),hl
	ld	a,CMD_ACK	;Send Acknowledgement for Skip
	ld	de,0
	call	Send_Header
	ld	de,VAR_OMITTED
	ld	c,9
	call	Bdos
	jp	SendLoop


;
;TI-83 told to quit sending...
;
S_Quit:	ld	a,CMD_ACK	;Send Acknowledgement for quit.
	ld	de,0
	call	Send_Header
	ld	de,ERR_ABORT
	ld	c,9	;Display 'Backup sent' message...
	jp	Bdos	;...and QUIT the program...


;
;End of recieving... (quit)
;
Send_End:	ld	a,CMD_EOT	;Give End Of Transmission signal
	ld	de,0
	call	Send_Header
Send_E_NoEOT:	call	Recv_Header	;Wait for the final ACK...
	cp	CMD_CHKERR
	jp	z,Chksum_Error
	cp	CMD_ACK
	jp	nz,Xmit_Error
	ld	de,DONE
	ld	c,9
	jp	Bdos	;(another 'RET' is in Send_Quit)



;
;This routine loads the file in FCB1 into memory.
;
;
Load_VarFile:	ld	a,(FCB1+9)	;Check if an extension...
	cp	" "
	jp	nz,LVF_DontExt
	ld	a,"8"	;If filename has no extension then
	ld	(FCB1+9),a	; start a search for the first one with
	ld	a,"3"	; the same name and a .83? extension.
	ld	(FCB1+10),a
	ld	a,"?"
	ld	(FCB1+11),a
	ld	de,FCB
	ld	c,26	;Set FCB as DTA...
	call	Bdos
	ld	de,FCB1
	ld	c,17	;Bdos call: "Search for first"
	call	Bdos	;(returns FCB to DTA => FCB)
	and	a
	jp	z,LVF_Found	;JUMP if found.
	ld	a,"2"	;Try the same name and a .82? extension.
	ld	(FCB1+10),a
LVF_DontExt:	ld	de,FCB
	ld	c,26	;Set FCB as DTA...
	call	Bdos
	ld	de,FCB1
	ld	c,17	;Bdos call: "Search for first"
	call	Bdos	;(returns FCB to DTA => FCB)
	and	a
	jp	nz,Load_ErrName	;If A>0 then *** File Not Found
;
LVF_Found:	ld	hl,FILENAME
	ld	de,LOADINGFNAME
	ld	bc,8
	ldir
	ld	hl,FILENAME+8
	ld	de,LOADINGFNAME+9
	ld	bc,3
	ldir
	ld	de,LOADINGTEXT
	ld	c,9
	call	Bdos
;
	ld	de,TEMPBUFFER
	ld	c,26	;Set DTA
	call	Bdos
	ld	de,FCB
	ld	c,15	;Create file
	call	Bdos
	and	a
	jp	nz,Load_ErrDisk
	ld	hl,0
	ld	(RECORDNR),hl
	ld	(RECORDNR+2),hl
	inc	hl
	ld	(RECORDLENGTH),hl
	ld	de,FCB
	ld	hl,FHDR_LENGTH + 2	;Also include datapart-length...
	ld	c,39	;Load fileheader
	call	Bdos
	and	a
	jp	nz,Load_ErrDisk
	ld	hl,(TEMPBUFFER + #35)
	ld	(DATALENGTH),hl
;
	ld	hl,TEMPBUFFER
	TestFileType "*"
	TestFileType "*"
	TestFileType "T"
	TestFileType "I"
	TestFileType "8"
	ld	a,(hl)
	inc	hl
	sub	"0"
	add	#80
	ld	(CURCALC),a
	TestFileType "*"
	TestFileType "*"
;
	ld	de,FILECOMMTEXT
	ld	c,9
	call	Bdos
	ld	a,13
	ld	(TEMPBUFFER + FHDR_LENGTH),a	;Put <CRLFLF> at the end
	ld	a,10	;               ; of the comment...
	ld	(TEMPBUFFER + FHDR_LENGTH + 1),a
	ld	(TEMPBUFFER + FHDR_LENGTH + 2),a
	ld	a,"$"
	ld	(TEMPBUFFER + FHDR_LENGTH + 3),a
	ld	hl,TEMPBUFFER+11
	ld	b,COMMENT_LEN
ASCII32_Loop:	ld	a,(hl)
	cp	32
	jp	nc,ASCII32_Next
	ld	a," "	;Replace ASCII-code < 32 by a space.
	ld	(hl),a
ASCII32_Next:	inc	hl
	djnz	ASCII32_Loop
	ld	de,TEMPBUFFER+11	;Display the comment...
	ld	c,9
	call	Bdos
;
	ld	de,Buffer_Start
	ld	c,26	;Set DTA
	call	Bdos
	ld	hl,(DATALENGTH)
	inc	hl	;Include checksum
	inc	hl
	ld	de,FCB	;Load data-part from file to mem...
	ld	c,39
	call	Bdos
	and	a
	jp	nz,Load_ErrDisk
	ld	hl,Buffer_Start
	ld	bc,(DATALENGTH)
	add	hl,bc
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	ld	hl,Buffer_Start
;        ------------------------------
	call	Check_Chksum
	jp	nz,Chksum_Error
;        ------------------------------
	ld	de,FCB
	ld	c,16	;Close file...
	call	Bdos
	and	a
	jp	nz,Load_ErrDisk
	ret




;
;==============================================================================
;= Recieve files... ===========================================================
;==============================================================================
;
;  START RECIEVING VAR-HEADER
;  ==========================
Recieve_File:	xor	a
	ld	(NR_OF_VARS),a	;Reset nr. of vars-counter
	ld	de,WAITINGTEXT
	ld	c,9
	call	Bdos
RecieveLoop:	call	Recv_Header	;Recieve first packetheader
	cp	CMD_EOT	;If End Of Transmission then end...
	jp	z,Recieve_End
	cp	CMD_VAR
	jp	z,RecvLopItsOk
	cp	CMD_SENDVAR
	jp	nz,Xmit_Error	;If not EOT or VAR-header then error.
RecvLopItsOk:	ld	a,(NR_OF_VARS)
	inc	a	;Add 1 to total amount of VARs...
	ld	(NR_OF_VARS),a
	call	Recv_2ndPart
	ld	hl,(LAST_RECVBUF)
	inc	hl	;Set the current VAR-type...
	inc	hl
	ld	a,(hl)
	ld	(CUR_VARTYPE),a
	ld	a,CMD_ACK	;Send ACK reply
	ld	de,0
	call	Send_Header
;          --------------------------
	call	RPrt_83Name	;Print VARname
;          --------------------------

;             START RECIEVING VAR-DATA
;             ========================
	ld	a,CMD_WAITING	;Send 'Waiting for data'-reply...
	ld	de,0
	call	Send_Header

	call	Recv_Header	;Recieve ACK confirmation
	cp	CMD_ACK
	jp	nz,Xmit_Error
	call	Recv_Header	;Recieve dataheader
	cp	CMD_DATA
	jp	nz,Xmit_Error
	call	Recv_2ndPart	;Recieve datablock

	ld	a,CMD_ACK
	ld	de,0
	call	Send_Header
	ld	a,(CUR_VARTYPE)
	cp	#13	;Check if the VAR was a backup...
	jp	nz,RecieveLoop	;If not, repeat the mainloop.

;             IF BACKUP, RECIEVE ANOTHER BLOCK
;             ================================
	call	Recv_Header
	cp	CMD_DATA	;#15 = Datapart
	jp	nz,Xmit_Error
	call	Recv_2ndPart	;Recieve datablock
	ld	a,CMD_ACK
	ld	de,0
	call	Send_Header
;
	call	Recv_Header
	cp	CMD_DATA	;#15 = Datapart
	jp	nz,Xmit_Error
	call	Recv_2ndPart	;Recieve datablock
	ld	a,CMD_ACK
	ld	de,0
	call	Send_Header
	ld	de,BACKUPRECVD
	ld	c,9
	call	Bdos
	;
	;Don't JP  RecieveLoop (no further data).
	;Just save immediately.
;
;End of recieving... (quit)
;
Recieve_End:	ld	a,CMD_ACK	;Give the final ACK answer
	ld	de,0
	call	Send_Header
;          --------------------------
	call	Save_VarFile
;          --------------------------
	ld	de,DONE
	ld	c,9
	jp	Bdos


;
;This routine asks for a filename (provided the nr. of VARs downloaded !=0).
; Next, it determines the extension, and then it saves the file to disk.
;
;
Save_VarFile:	ld	a,(NR_OF_VARS)	;If no VARs received then end.
	cp	1
	ret	c
;
	ld	a,(NR_OF_VARS)	;Determine extension
	ld	hl,FILEEXT_83G	;
	cp	2	;If >1 files then .83G
	jp	nc,Save_ExtCont
	ld	a,(CUR_VARTYPE)	;If 1 file search for right extension
	ld	b,a	;
	add	a,a	;A=Ax3
	add	a,b
	ld	c,a
	ld	b,0
	ld	hl,FILEEXT_LIST
	add	hl,bc
Save_ExtCont:	ld	a,(hl)
	ld	(FILENAME+8),a
	ld	(INPUT_FN_EXT),a
	inc	hl
	ld	a,(CURCALC)
	sub	#80
	add	"0"
	ld	(FILENAME+9),a
	ld	(INPUT_FN_EXT+1),a
	ld	(FILEHEADER+5),a
	inc	hl	;Now the filename is in the FCB...
	ld	a,(hl)	;...let's rock & save!!!
	ld	(FILENAME+10),a
	ld	(INPUT_FN_EXT+2),a
;
	ld	de,INPUT_FN_TXT
	ld	c,9
	call	Bdos	;Let the user input a filename
	ld	de,FN_INPUTBUF
	ld	c,10
	call	Bdos
	ld	e,10
	ld	c,2
	call	Bdos
	ld	a,(FN_INPUTBUF+1)
	and	a
	jp	z,Save_NoSave
	ld	hl,FILENAME
	ld	de,FILENAME+1
	ld	bc,8-1	;Move the input filename to the FCB
	ld	(hl)," "
	ldir
	ld	hl,FN_INPUTBUF+2
	ld	de,FILENAME
	ld	b,a
Save_InpLoop:	ld	a,(hl)
	ld	(de),a
	inc	hl
	inc	de
	djnz	Save_InpLoop
;
	ld	de,FILEHEADER	;Headertype (TI-83 or 82) is set above...
	ld	c,26	;Set DTA
	call	Bdos
	ld	de,FCB
	ld	c,22	;Create file
	call	Bdos
	and	a
	jp	nz,Save_ErrName
	ld	hl,0
	ld	(RECORDNR),hl
	ld	(RECORDNR+2),hl
	inc	hl
	ld	(RECORDLENGTH),hl
	ld	de,FCB
	ld	hl,FHDR_LENGTH
	ld	c,38	;Write fileheader
	call	Bdos
	and	a
	jp	nz,Save_ErrDisk
;
	ld	de,Buffer_Start-2
	ld	c,26	;Set DTA
	call	Bdos
	ld	hl,(RECVBUFFER)
	ld	de,Buffer_Start
	and	a	;Calculate datalength & checksum
	sbc	hl,de
	ld	(Buffer_Start-2),hl
	ld	b,h
	ld	c,l
	ld	hl,Buffer_Start
;        ------------------------------
	call	Make_Chksum
;        ------------------------------
	ld	hl,(RECVBUFFER)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,(Buffer_Start-2)
	inc	hl
	inc	hl	;datalength +4: 2=LENGTH byte, 2=CHECKSUM
	inc	hl
	inc	hl
	ld	de,FCB	;Write datapart + checksum to file
	ld	c,38
	call	Bdos
	and	a
	jp	nz,Save_ErrDisk
	ld	de,FCB
	ld	c,16	;Close file...
	call	Bdos
	and	a
	jp	nz,Save_ErrDisk
;
	ld	de,SAVE_SUCCESS
	ld	c,9	;Notify user of success and end
	jp	Bdos


;




;
;This routine detracts the VAR-name, converts it to plain ASCII and puts
; it in a string. Next, it puts the VAR-type in the string.
; And finally it prints the string on the screen.
;
; RECEIVING
;
RPrt_83Name:	ld	hl,RECIEVE_VAR	;Set right text (:Recieving)
	ld	de,VAR_PRINT
	ld	bc,11
	ldir
	ld	hl,(LAST_RECVBUF)
	inc	hl	;
	inc	hl
	inc	hl
	ld	de,VAR_NAME
	ld	bc,8	;Display variable-name
	ldir
	call	ConvName
	call	Put_VARtype
	ld	de,VAR_PRINT
	ld	c,9
	jp	Bdos
;
; SENDING
;
SPrt_83Name:	ld	hl,SEND_VAR	;Set right text (:Send)
	ld	de,VAR_PRINT
	ld	bc,11
	ldir
	ld	hl,(RECVBUFFER)
	ld	bc,5
	add	hl,bc
	ld	de,VAR_NAME
	ld	bc,8
	ldir
	call	ConvName
	call	Put_VARtype
	ld	de,VAR_PRINT
	ld	c,9
	jp	Bdos



;
;Put the VAR-type (REAL, WINDW, PRGM) in the string which
; will be displayed.
;
Put_VARtype:	ld	a,(CUR_VARTYPE)	;Find vartype in list
	ld	b,a	;
	add	a,a	;A=Ax5
	add	a,a
	add	a,b
	ld	c,a
	ld	b,0
	ld	hl,TYPESLIST
	add	hl,bc
	ld	de,VAR_TYPE
	ld	bc,5
	ldir
	ret



;
;This routine converts some TI-83 filenames (with TI-specific charsets)
; to a bit more 'readable' ones...
;
ConvName:	ld	a,(CUR_VARTYPE)
	cp	#0F
	jp	nc,ConvWinTbl
;             ----------------------
	ld	hl,VAR_NAME
	ld	bc,VAR_NAME+8
ConvNameLoop:	ld	a,(hl)
	cp	0	;Convert 0 to space...
	jp	z,ConvZero
	cp	#5B	;Theta-character...
	jp	z,ConvTheta
	cp	#5C	;Matrix: [A], [B] etc.
	jp	z,ConvList
	cp	#5D	;List: L1, L2, lABC, etc.
	jp	z,ConvList
	cp	#5E	;Equation: Y1, X1t, r1, v, etc.
	jp	z,ConvEqu
	cp	#60	;Picture: Pic0, Pic1, etc.
	jp	z,ConvPic
	cp	#61	;GBD: GBD0, GBD1, etc.
	jp	z,ConvGDB
	cp	#AA	;String: Str0, Str1, etc.
	jp	z,ConvStr
ConvNameEnd:	inc	hl
	push	hl
	sbc	hl,de
	pop	hl
	jp	c,ConvNameLoop
	ret

;
;SUBROUTINES FOR NAMECONVERT-ROUTINE
;
ConvZero:	ld	(hl)," "	;Space
	jp	ConvNameEnd
;
ConvTheta:	ld	(hl),233	;Theta-sign
	jp	ConvNameEnd
;
ConvList:	ld	(hl),"l"
	inc	hl
	ld	a,(hl)
	cp	#10	;Check if value <#10...
	jp	nc,ConvNameEnd
	dec	hl	;If so, then L1/2/3/etc-list...
	ld	(hl),"L"
	inc	hl
	add	a,"1"
	ld	(hl),a
	jp	ConvNameEnd
;
ConvMatrix:	ld	(hl),"["
	inc	hl
	ld	a,(hl)
	add	a,"A"
	ld	(hl),a
	inc	hl
	ld	(hl),"]"
	jp	ConvNameEnd
;
ConvEqu:	inc	hl
	ld	a,(hl)
	dec	hl
	bit	4,a
	jp	nz,ConvEqu_Y
	bit	5,a
	jp	nz,ConvEqu_XYt
	bit	6,a
	jp	nz,ConvEqu_r
ConvEqu_UVW:	cp	#81
	jp	z,ConvEqu_Vn
	jp	nc,ConvEqu_Wn
	ld	(hl),"u"
	inc	hl
	ld	(hl)," "
	jp	ConvNameEnd
ConvEqu_Vn:	ld	(hl),"v"
	inc	hl
	ld	(hl)," "
	jp	ConvNameEnd
ConvEqu_Wn:	ld	(hl),"w"
	inc	hl
	ld	(hl),0
	jp	ConvNameEnd
ConvEqu_r:	ld	(hl),"r"
	inc	hl
	and	#0F
	add	a,"1"
	ld	(hl),a
	jp	ConvNameEnd
ConvEqu_XYt:	and	#0F	;Resets C so RRA isn't SRL A
	rra
	jp	c,ConvEqu_Yt
	ld	(hl),"X"
	inc	hl
	add	a,"1"
	ld	(hl),a
	jp	ConvNameEnd
ConvEqu_Yt:	ld	(hl),"Y"
	inc	hl
	add	a,"1"
	ld	(hl),a
	jp	ConvNameEnd
ConvEqu_Y:	ld	(hl),"Y"
	inc	hl
	cp	#19
	jp	z,ConvEqu_Y0
	and	#0F
	add	a,"1"
	ld	(hl),a
	jp	ConvNameEnd
ConvEqu_Y0:	ld	(hl),"0"
	jp	ConvNameEnd
;
ConvPic:	ld	(hl),"P"
	inc	hl
	ld	a,(hl)
	ld	(hl),"i"
	inc	hl
	ld	(hl),"c"
	inc	hl
	cp	#09
	jp	z,ConvPic0
	add	a,"1"
	ld	(hl),a
	jp	ConvNameEnd
ConvPic0:	ld	(hl),"0"
	jp	ConvNameEnd
;
ConvGDB:	ld	(hl),"G"
	inc	hl
	ld	a,(hl)
	ld	(hl),"D"
	inc	hl
	ld	(hl),"B"
	inc	hl
	cp	#09
	jp	z,ConvGDB0
	add	a,"1"
	ld	(hl),a
	jp	ConvNameEnd
ConvGDB0:	ld	(hl),"0"
	jp	ConvNameEnd
;
ConvStr:	ld	(hl),"S"
	inc	hl
	ld	a,(hl)
	ld	(hl),"t"
	inc	hl
	ld	(hl),"r"
	inc	hl
	cp	#09
	jp	z,ConvStr0
	add	a,"1"
	ld	(hl),a
	jp	ConvNameEnd
ConvStr0:	ld	(hl),"0"
	jp	ConvNameEnd
;
ConvWinTbl:	ld	hl,C_WINDOW
	cp	#10
	jp	c,ConvWinTbl_C
	ld	hl,C_RCLWINDOW
	jp	z,ConvWinTbl_C
	cp	#12
	ld	hl,C_TBLSET
	jp	c,ConvWinTbl_C
	ld	hl,C_BACKUP
ConvWinTbl_C:	ld	bc,9
	ld	de,VAR_NAME
	ldir
	ret




;
;==============================================================================
;= Some data transfer subroutines =============================================
;==============================================================================
;
; - Recv_BCBytes: Recieves BC bytes from the TI calculator
; - Send_BCBytes: Sends BC bytes to the TI calculator
; - Recv_Header:  Recieves the header of a packet (1st 4 bytes)
; - Recv_2ndPart: Recieves the rest of a packet and checks checksum
; - Check_Chksum: Calculates & compares the checksum
;
;
;This routine recieves a 4-byte header from the TI-83.
; The command-byte is passed in A and the LENGTH-byte is passed in DE.
; It will give an error if the packet isn't coming from a TI-83.
;
Recv_Header:	call	RecvByte
	ld	(CURCALC),a
	and	#FE	;Check if it's a TI-83 or 82...
	cp	#82
	jp	nz,Xmit_NoTI83
	call	RecvByte
	ld	d,a
	call	RecvByte
	ld	e,a
	call	RecvByte
	ld	c,d
	ld	d,a
	ld	a,c
	ret


;
;This routine loads the second part of a packet into the (RECVBUFFER),
; and it includes the length of the packet as the 1st 2 bytes.
; LAST_RECVBUF is set to the start of the packet (so it skips those 2 bytes).
; Afterwards, the checksum is checked and the RECVBUFFER pointer is updated.
;
;The length of the second part must be DE.
;
;
Recv_2ndPart:	ld	hl,(RECVBUFFER)
	ld	b,d
	ld	c,e
	push	bc
	ld	(hl),c
	inc	hl
	ld	(hl),b
	inc	hl
	ld	(LAST_RECVBUF),hl
	inc	bc
	inc	bc
	call	Recv_BCBytes
	dec	hl
	ld	d,(hl)
	dec	hl
	ld	e,(hl)
	ld	(RECVBUFFER),hl
	ld	hl,(LAST_RECVBUF)
	pop	bc
	call	Check_Chksum
	ret	nc
	jp	Chksum_Error



;
;This routine sends 4 bytes with ID A and length DE to the TI-calculator
;
Send_Header:	push	af
	ld	a,(CURCALC)	;Computer -> TI-83/82
	and	#0F
	call	SendByte
	pop	af
	call	SendByte
	ld	a,e
	call	SendByte
	ld	a,d
	jp	SendByte



;
;This routine sends an entire packet (type A) to the TI-83...
;
Send_Packet:	ld	hl,(RECVBUFFER)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	(RECVBUFFER),hl
	call	Send_Header
	ld	b,d
	ld	c,e
	ld	hl,(RECVBUFFER)
	push	hl
	push	bc
	ld	hl,(RECVBUFFER)
	call	Send_BCBytes
	ld	(RECVBUFFER),hl
	pop	bc
	pop	hl
	call	Make_Chksum
	ld	a,e
	call	SendByte
	ld	a,d
	jp	SendByte



;
;This routine recieves BC bytes from the TI-calculator.
;Input: HL=pointer to header, BC=Nr. of bytes to recieve
;Output: HL=updated pointer
;
Recv_BCBytes:	dec	bc
	inc	b
	inc	c	;Preloop-sequence for fast loop...
	ld	a,c
	ld	c,b
	ld	b,a
Recv_BCBLoop:	push	bc
	call	RecvByte
	ld	(hl),a
	inc	hl
	pop	bc
	djnz	Recv_BCBLoop
	dec	c
	jp	nz,Recv_BCBLoop
	ret



;
;This routine sends BC bytes to the TI-calculator.
;Input: HL=pointer to header, BC=Nr. of bytes to send
;Output: HL=updated pointer
;
Send_BCBytes:	dec	bc
	inc	b
	inc	c	;Preloop-sequence for fast loop...
	ld	a,c
	ld	c,b
	ld	b,a
Send_BCBLoop:	ld	a,(hl)
	inc	hl
	push	bc
	call	SendByte
	pop	bc
	djnz	Send_BCBLoop
	dec	c
	jp	nz,Send_BCBLoop
	ret



;
;This routine calculates the checksum from HL to HL+BC,
; and compares it with the value in DE...
;Result: Z if checksum is ok, NZ if checksum is incorrect
;        HL=updated pointer
;
Check_Chksum:	dec	bc
	inc	b
	inc	c	;Preloop-sequence for fast loop...
	ld	a,c
	ld	c,b
	ld	b,a
ChkChk_Loop:	ld	a,e
	sub	(hl)	;SUB  DE,(HL)
	inc	hl
	ld	e,a
	ld	a,d
	sbc	a,0
	ld	d,a
	djnz	ChkChk_Loop
	dec	c
	jp	nz,ChkChk_Loop
	ld	a,d
	or	e
	ret



;
;This routine generates a checksum for the area from HL to HL+BC,
;Result: DE=Generated checksum, HL=updated pointer
;
Make_Chksum:	dec	bc
	inc	b
	inc	c	;Preloop-sequence for fast loop...
	ld	a,c
	ld	c,b
	ld	b,a
	ld	de,0
MakeChk_Loop:	ld	a,e
	add	a,(hl)	;ADD  DE,(HL)
	inc	hl
	ld	e,a
	ld	a,d
	adc	a,0
	ld	d,a
	djnz	MakeChk_Loop
	dec	c
	jp	nz,MakeChk_Loop
	ret


;
;==============================================================================
;= Transmission routines ======================================================
;==============================================================================
;
;The following routines are included:
; - RecvByte: Recieves a byte from the TI calculator
; - SendByte: Sends a byte to the TI calculator
;
;
;This routine recieves a byte from the TI-83...
;Result: A=recieved byte.
;        Carry=Error. If error then:
;         - if A=0 / the Z flag is set then the transmission was aborted.
;         - if A=255 / the Z flag is not set then a transmission error occurred.
;
;Changes: AF, BC
;
;!!! NEVER USE DE !!! otherwise adapt other routines.
;
RecvByte:	di
	Send_TI	s11	;Make sure pins are set right to recieve.
	ld	b,8	;B=bitcounter, C=byte to be filled...
	ld	c,0
RecvByteLoop:	SetTestESC	;Prepare ESC readout & Joy readout
	Recv_TIa
RecvByte_Lop:	DoTestESC
	jp	z,Xmit_Abort
	Recv_TIb
	and	r11
	cp	r11
	jp	z,RecvByte_Lop
	cp	r01
	jp	z,Recv_0bit	;Some kind of CALL...
	jp	nc,Recv_1bit
	jp	c,Xmit_Error
RecvByteCont:	djnz	RecvByteLoop	;...which returns HERE!!!
	ei
	ld	a,c
	ret



;Recieve a 0-bit.
;
Recv_0bit:	rrc	c	;Leave bit 0...
	Send_TI	s10
	SetTestESC	;Prepare ESC readout & Joy readout
	Recv_TIa
Recv_0_Loop:	DoTestESC
	jp	z,Xmit_Abort
	Recv_TIb
	and	r10
	jp	z,Recv_0_Loop
	Send_TI	s11
	jp	RecvByteCont


;Recieve a 1-bit.
;
Recv_1bit:	rrc	c
	set	7,c	;Set bit to 1...
	Send_TI	s01
	SetTestESC	;Prepare ESC readout & Joy readout
	Recv_TIa
Recv_1_Loop:	DoTestESC
	jp	z,Xmit_Abort
	Recv_TIb
	and	r01
	jp	z,Recv_1_Loop
	Send_TI	s11
	jp	RecvByteCont




;
;This routine sends a byte to the TI-83...
;Input: A=byte to send...
;Result: Carry=Error. If error then:
;         - if A=0 / the Z flag is set then the transmission was aborted.
;         - if A=255 / the Z flag is not set then a transmission error occurred.
;
;Changes: AF, BC
;
;
SendByte:	ld	b,8
	ld	c,a
	di
SendByteLoop:	rrc	c
	jp	nc,Send_0bit
	jp	c,Send_1bit
SendByteCont:	djnz	SendByteLoop
	ei
	ret



;Send a 0-bit.
;
Send_0bit:	Send_TI	s01
	SetTestESC	;Prepare ESC readout & Joy readout
	Recv_TIa
Send_0_Loop1:	DoTestESC
	jp	z,Xmit_Abort
	Recv_TIb
	and	r01
	jp	nz,Send_0_Loop1
	Send_TI	s11
	SetTestESC	;Prepare ESC readout & Joy readout
	Recv_TIa
Send_0_Loop2:	DoTestESC
	jp	z,Xmit_Abort
	Recv_TIb
	and	r01
	jp	z,Send_0_Loop2
	jp	SendByteCont



;Send a 1-bit.
;
Send_1bit:	Send_TI	s10
	SetTestESC	;Prepare ESC readout & Joy readout
	Recv_TIa
Send_1_Loop1:	DoTestESC
	jp	z,Xmit_Abort
	Recv_TIb
	and	r10
	jp	nz,Send_1_Loop1
	Send_TI	s11
	SetTestESC	;Prepare ESC readout & Joy readout
	Recv_TIa
Send_1_Loop2:	DoTestESC
	jp	z,Xmit_Abort
	Recv_TIb
	and	r10
	jp	z,Send_1_Loop2
	jp	SendByteCont



;
;==============================================================================
;= Other routines =============================================================
;==============================================================================
;
;Error routines: Xmit_Abort & Xmit_Error.
;
Xmit_Error:	ld	de,ERR_XMIT	;Transmission error...
	ld	c,9
	call	Bdos
	ld	sp,(save_SP)
	ret

Xmit_Abort:	ld	de,ERR_ABORT	;User aborted (pressed ESC)...
	ld	c,9
	call	Bdos
	ld	sp,(save_SP)
	ret

Chksum_Error:	ld	de,ERR_CHECKSUM	;Checksum error...
	ld	c,9
	call	Bdos
	ld	sp,(save_SP)
	ret

Xmit_NoTI83:	ld	de,ERR_CALCTYPE	;Transmission not from a TI-88 or 82...
	ld	c,9
	call	Bdos
	ld	sp,(save_SP)
	ret

Load_ErrName:	ld	de,ERR_FNFOUND	;File not found (loadroutines)...
	ld	c,9
	call	Bdos
	ld	sp,(save_SP)
	ret

Load_ErrDisk:	ld	de,ERR_DISK	;Disk-error (loadroutines)...
	ld	c,9
	call	Bdos
	ld	sp,(save_SP)
	ret

;
;The following error-routines do not terminate the program, they RET instead.
;
Save_NoSave:	ld	de,NOTSAVED_MSG	;Not saved because (no filename)...
	ld	c,9
	jp	Bdos
;
Save_ErrName:	ld	de,ERR_SAVENAME	;Invalid filename (saveroutines)...
	ld	c,9
	jp	Bdos
;
Save_ErrDisk:	ld	de,ERR_DISKSAVE	;Disk error, not saved (saveroutines)...
	ld	c,9
	jp	Bdos

;
;====================================================
;
;
;PrintHex
;Deze routine zet een hexadecimaal getal om in tekst en
;zet het vervolgens op het scherm.
;Invoer: A=Getal
;
PrintHex:	push	af,bc,de,hl
	push	af
	and	#F0
	rrca
	rrca
	rrca
	rrca
	ld	c,a
	ld	b,0
	ld	hl,HexTabel
	add	hl,bc
	ld	e,(hl)	;Let op! E voor de BDOS maar A voor de BIOS
	ld	c,2
	call	Bdos

	pop	af
	and	#0F
	ld	c,a
	ld	b,0
	ld	hl,HexTabel
	add	hl,bc
	ld	e,(hl)	;Let op! E voor de BDOS maar A voor de BIOS
	ld	c,2
	call	Bdos
	pop	hl,de,bc,af
	ret


;Data's
;
HexTabel:	DB	"0123456789ABCDEF"







;
;==============================================================================
;= Data =======================================================================
;==============================================================================
;
;Data's...
;
;Diskvariables...
;
FCB:	DB	0
FILENAME:	DB	"filename83x"
BLOCK:	DW	#00
RECORDLENGTH:	DW	#0001
FILELENGTH:	DW	#0000,#0000
SYSTEMVAR:	DS	13,0
RECORDNR:	DW	#0000,#0000


;
;Text
;
INTROTEXT:	DB	12,"MSX <-> TI-83 terminal",13,10,10
	DB	"(c) 1999  Grauw / Datax",13,10,10,10,"$"
WAITINGTEXT:	DB	"Waiting...",13,10,"$"
DONE:	DB	10,"Done!!!",13,10,10,"$"
VAR_PRINT:	DB	"Recieving: "
VAR_NAME:	DB	"xxxxxxxx  ("
VAR_TYPE:	DB	"xxxxx)",13,10,"$"
RECIEVE_VAR:	DB	"Recieving: "
SEND_VAR:	DB	"Sending: ",0,0
BACKUPRECVD:	DB	"Backup recieved.",13,10,"$"
BACKUPSENT:	DB	"Backup sent.",13,10,"$"
VAR_OMITTED:	DB	30,"Skipped",13,10,"$"
;
LOADINGTEXT:	DB	"Loading file: "
LOADINGFNAME:	DB	"filename.ext",13,10
	DB	"==========================",13,10,"$"
FILECOMMTEXT:	DB	"File comment: $"
SAVE_SUCCESS:	DB	"Saving succeeded!!!",13,10,"$"
INPUT_FN_TXT:	DB	10,"Input filename to save to:         ."
INPUT_FN_EXT:	DB	"83x",29,29,29,29,29,29,29,29,29,29,29,29,"$"
NOTSAVED_MSG:	DB	"!!! Recieved files not saved !!!",13,10,"$"
FN_INPUTBUF:	DB	8,0,"xxxxxxxx"	;Filename input buffer for 8 characters
;
ERR_XMIT:	DB	7,"*** Transmission error",13,10,"$"
ERR_CHECKSUM:	DB	7,"*** Checksum error",13,10,"$"
ERR_ABORT:	DB	7,"*** Aborted by user",13,10,"$"
ERR_CALCTYPE:	DB	7,"*** Calculator not supported",13,10,"$"
ERR_INV_COML:	DB	7,"*** Invalid commandline",13,10,"$"
ERR_DISK:	DB	7,"*** Disk error",13,10,"$"
ERR_DISKSAVE:	DB	7,"*** Disk error, file not saved",13,10,"$"
ERR_FNFOUND:	DB	7,"*** File not found",13,10,"$"
ERR_NOTI83:	DB	7,"*** No TI-83 file",13,10,"$"
ERR_SAVENAME:	DB	7,"*** Invalid filename, file not saved",13,10,"$"
;


;
;File extensions...
;
FILEEXT_83G:	DB	"83G"	;Group file
FILEEXT_LIST:	DB	"83N"	;Real number
	DB	"83L"	;List
	DB	"83M"	;Matrix
	DB	"83E"	;Equation
	DB	"83S"	;String
	DB	"83P"	;Program
	DB	"83P"	;Protected Program
	DB	"83I"	;Picture
	DB	"83D"	;Graphics Database (GDB)
	DB	"83 "	;
	DB	"83 "	;
	DB	"83 "	;
	DB	"83C"	;Complex number
	DB	"83 "	;
	DB	"83 "	;
	DB	"83R"	;Window
	DB	"83R"	;RclWindow
	DB	"83R"	;TblSet
	DB	"83 "	;
	DB	"83B"	;Backup

;
;.83?-fileformat strings...
;
FILEHEADER:	DB	"**TI83**",26,10,0
FHDR_COMMENT:	DB	"  -=| TransMSX  (c)1999 Grauw/Datax |=-   "
FILEDATAPART:
FHDR_LENGTH:	equ	FILEDATAPART -FILEHEADER	;File header=53 (=#35) bytes long...
COMMENT_LEN:	equ	FILEDATAPART - FHDR_COMMENT

;
C_WINDOW:	DB	"Window   "
C_RCLWINDOW:	DB	"RclWindow"
C_TBLSET:	DB	"TblSet   "
C_BACKUP:	DB	"Backup   "

;
TYPESLIST:	DB	"REAL",0	;#00
	DB	"LIST",0	;#01
	DB	"MATRX"	;#02
	DB	"EQU",0,0	;#03
	DB	"STRNG"	;#04
	DB	"PRGM",0	;#05
	DB	"PRGM."	;#06
	DB	"PIC",0,0	;#07
	DB	"GDB",0,0	;#08
	DB	"#09  "	;#09
	DB	"#0A  "	;#0A
	DB	"#0B  "	;#0B
	DB	"CPLX",0	;#0C
	DB	"#0D  "	;#0D
	DB	"#0E  "	;#0E
	DB	"WINDW"	;#0F
	DB	"ZSTO",0	;#10
	DB	"TABLE"	;#11
	DB	"#12  "	;#12
	DB	"BACK",0	;#13


;
;Variables...
;
save_SP:	DW	#0000	;Save of 'original' stackpointer
NR_OF_VARS:	DB	0	;Nr. of VARs loaded...
CUR_VARTYPE:	DB	0	;The current VAR-type...
;
DATALENGTH:	DW	#0000	;Total length of packets (save & load).
DATAEND:	DW	#4000	;Pointer to end of buffer.
RECVBUFFER:	DW	#4000	;Pointer to current pos. in the buffer.
LAST_RECVBUF:	DW	#4000	;Pointer to last recieved packet in buffer.
;
CURCALC:	DB	0	;Type of the connected calculator...
;
TEMPBUFFER:	DS	256,0	;Temporary recieve-buffer
;


Einde:
;
;

